import unittest

from odoo_env_config.api import Env, OdooCliFlag
from odoo_env_config.section.http_section import HttpOdooConfigSection


class TestHttpOdooConfigSection(unittest.TestCase):
    def test_default(self):
        conf = HttpOdooConfigSection()
        self.assertEqual(OdooCliFlag({"no-http": True}), conf.to_values())
        self.assertIsNone(conf.interface)
        self.assertEqual(0, conf.port)
        self.assertFalse(conf.enable)
        self.assertEqual(0, conf.longpolling_port)

    def test_disable(self):
        conf = HttpOdooConfigSection()
        conf.enable = True
        self.assertEqual(OdooCliFlag(), conf.to_values())
        self.assertIsNone(conf.interface)
        self.assertEqual(0, conf.port)
        self.assertTrue(conf.enable)
        self.assertEqual(0, conf.longpolling_port)

    def test_global_http_key(self):
        conf = HttpOdooConfigSection().init(
            Env(
                {
                    "LONGPOLLING_PORT": "4040",
                    "HTTP_INTERFACE": "0.1.2.3",
                    "HTTP_PORT": "8080",
                    "HTTP_ENABLE": "True",
                }
            )
        )
        self.assertEqual("0.1.2.3", conf.interface)
        self.assertEqual(8080, conf.port)
        self.assertTrue(conf.enable)
        self.assertEqual(4040, conf.longpolling_port)
        self.assertDictEqual(
            OdooCliFlag({"longpolling-port": 4040, "http-port": 8080, "http-interface": "0.1.2.3"}), conf.to_values()
        )

    def test_enable(self):
        conf = HttpOdooConfigSection().init(
            Env(
                {
                    "HTTP_ENABLE": "True",
                }
            )
        )
        self.assertEqual(OdooCliFlag(), conf.to_values())
        self.assertIsNone(conf.interface)
        self.assertEqual(0, conf.port)
        self.assertTrue(conf.enable)
        self.assertEqual(0, conf.longpolling_port)
        conf = HttpOdooConfigSection().init(
            Env(
                {
                    "HTTP_ENABLE": "False",
                }
            )
        )
        self.assertEqual(OdooCliFlag({"no-http": True}), conf.to_values())
        self.assertIsNone(conf.interface)
        self.assertEqual(0, conf.port)
        self.assertFalse(conf.enable)
        self.assertEqual(0, conf.longpolling_port)



"""

"""
import os
from typing import Callable, Dict, List, Set, Type

from . import api, mappers
from .section import (
    AddonsPathConfigSection,
    DatabaseOdooConfigSection,
    HttpOdooConfigSection,
    LimitOdooConfigSection,
    LoggerSection,
    MiscSection,
    ServerWideModuleConfigSection,
    TestOdooConfigSection,
    UpdateInstallSection,
    WorkersOdooConfigSection,
)

CONVERTER: Set[Type[api.EnvConverter]] = {
    AddonsPathConfigSection,
    DatabaseOdooConfigSection,
    HttpOdooConfigSection,
    LimitOdooConfigSection,
    LoggerSection,
    MiscSection,
    TestOdooConfigSection,
    UpdateInstallSection,
    ServerWideModuleConfigSection,
    WorkersOdooConfigSection,
}
MAPPER: Set[Callable[[api.Env], api.Env]] = {
    mappers.compatibility,
    mappers.clevercloud_postgresql,
    mappers.redis_session,
    mappers.clevercloud_cellar,
    mappers.queue_job,
}


def apply_mapper(env: api.Env) -> api.Env:
    """
    Apply the MAPPER on `env` and return a new `api.Env` without mutate `env`
    Args:
        env: The env to map

    Returns:
        A new `api.Env` with all MAPPER applied on.
    """
    curr_env = env.copy()
    for mapper in MAPPER:
        curr_env = mapper(curr_env)
    return curr_env


def apply_converter(env: api.Env) -> api.OdooCliFlag:
    """
    Apply the CONVERTER to extract the value of `env` and return all the Odoo args founded
    Args:
        env: The env to convert to OdooCliFlag

    Returns:
        All the args found by the CONVERTER
    """
    store_values = api.OdooCliFlag()
    for converter in CONVERTER:
        store_values.update(converter().init(env).to_values())
    return store_values


def env_to_odoo_args(extra_env: Dict[str, str] = None) -> List[str]:
    """
    Entrypoint of this library
    Convert [os.environ][os.environ] to a odoo args valid.
    See Also
         The env to args [converter][odoo_env_config.option_group]
         The speccific cloud [env mapper][odoo_env_config.env_mapper]
    Examples
         >>> import odoo
         >>> odoo.tools.config.parse_args(env_to_odoo_args())
         >>> odoo.tools.config.save()
    Returns:
         A list with args created from Env
    """
    curr_env = api.Env(os.environ)
    curr_env.update(extra_env or {})
    curr_env = apply_mapper(env=curr_env)
    store_values = apply_converter(curr_env)
    return api.dict_to_odoo_args(store_values)



from typing import List

from addons_installer import addons_installer
from typing_extensions import Self

from .. import api


class AddonsPathConfigSection(api.EnvConverter):
    def __init__(self):
        super().__init__()
        self.registry = addons_installer.AddonsFinder()
        self.addons_path: List[str] = []

    def init(self, curr_env: api.Env) -> Self:
        result = self.registry.parse_env(env_vars=curr_env)
        self.addons_path = [r.addons_path for r in result]
        return self

    def to_values(self) -> api.OdooCliFlag:
        res = super().to_values()
        res.set("addons-path", self.addons_path)
        return res



from typing import Union

from typing_extensions import Self

from .. import api


class HttpOdooConfigSection(api.EnvConverter):
    def __init__(self):
        super().__init__()
        self.enable: bool = False
        self.interface: Union[str, None] = None
        self.port: int = 0
        self.longpolling_port: int = 0

    def init(self, curr_env: api.Env) -> Self:
        self.enable = curr_env.get_bool("HTTP_ENABLE", default=True)
        self.interface = curr_env.get("HTTP_INTERFACE")
        self.port = curr_env.get_int("HTTP_PORT")
        self.longpolling_port = curr_env.get_int("LONGPOLLING_PORT")
        return self

    def to_values(self) -> api.OdooCliFlag:
        res = super().to_values()
        if not self.enable:
            return res.set("no-http", True)

        res.set("http-interface", self.interface)
        res.set("http-port", self.port)
        res.set("longpolling-port", self.longpolling_port)
        return res



# -*- coding: utf8 -*-
import unittest

from odoo_env_config import entry


class TestOdooConfig(unittest.TestCase):
    def test_no_config(self):
        config = entry.env_to_odoo_args()
        self.assertFalse(config)

    def test_args_database(self):
        db_host = "my-host.com"
        db_name = "my-db"
        db_port = str(5253)
        db_user = "my-user"
        db_password = "py-password"
        extra_env = {
            "POSTGRESQL_ADDON_DB": db_name,
            "POSTGRESQL_ADDON_HOST": db_host,
            "POSTGRESQL_ADDON_PORT": db_port,
            "POSTGRESQL_ADDON_USER": db_user,
            "POSTGRESQL_ADDON_PASSWORD": db_password,
        }
        args = entry.env_to_odoo_args(extra_env)
        self.assertSetEqual(
            {
                "--db_host=" + db_host,
                "--db_port=" + db_port,
                "--db_user=" + db_user,
                "--db_password=" + db_password,
                "--database=" + db_name,
            },
            set(args),
        )

    def test_args_direct_database(self):
        """
        Assert Mapper for direct acces postgres provide by clevercloud
        """
        db_host = "my-host.com"
        db_name = "my-db"
        db_port = str(5253)
        db_user = "my-user"
        db_password = "py-password"
        extra_env = {
            "POSTGRESQL_ADDON_DB": db_name,
            "POSTGRESQL_ADDON_DIRECT_HOST": db_host,
            "POSTGRESQL_ADDON_DIRECT_PORT": db_port,
            "POSTGRESQL_ADDON_USER": db_user,
            "POSTGRESQL_ADDON_PASSWORD": db_password,
        }
        args = entry.env_to_odoo_args(extra_env)
        self.assertSetEqual(
            {
                "--db_host=" + db_host,
                "--db_port=" + db_port,
                "--db_user=" + db_user,
                "--db_password=" + db_password,
                "--database=" + db_name,
            },
            set(args),
        )



import unittest
from tests import _decorators
from ._helpers import create_config, assertParser


@_decorators.SkipUnless.env_odoo
class AllOdooVersionTest(unittest.TestCase):
    def test_db(self):
        parser = create_config()
        self.assertFalse(parser.getboolean("options", "db_host"))
        self.assertEqual(64, parser.getint("options", "db_maxconn"))
        self.assertFalse(parser.getboolean("options", "db_name"))
        self.assertFalse(parser.getboolean("options", "db_password"))
        self.assertFalse(parser.getboolean("options", "db_port"))
        self.assertEqual("prefer", parser.get("options", "db_sslmode"))
        self.assertEqual("template0", parser.get("options", "db_template"))
        self.assertFalse(parser.getboolean("options", "db_user"))
        self.assertEqual("", parser.get("options", "dbfilter"))

    def test_db_default(self):
        parser = create_config()
        assertParser(
            self,
            parser,
            {
                "db_host": False,
                "db_maxconn": 64,
                "db_name": False,
                "db_password": False,
                "db_port": False,
                "db_sslmode": "prefer",
                "db_template": "template0",
                "db_user": False,
                "dbfilter": "",
            },
        )

    def test_db_default15(self):
        parser = create_config()
        assertParser(
            self,
            parser,
            {
                "db_host": False,
                "db_maxconn": 64,
                "db_name": False,
                "db_password": False,
                "db_port": False,
                "db_sslmode": "prefer",
                "db_template": "template0",
                "db_user": False,
                "dbfilter": "",
            },
        )

    def test_db_profile_clever(self):
        parser = create_config(["test_db_clever"])
        assertParser(
            self,
            parser,
            {
                "db_host": "my_db_host",
                "db_maxconn": 64,
                "db_name": "my_db_name",
                "db_password": "my_db_password",
                "db_port": 1234,
                "db_sslmode": "prefer",
                "db_template": "template0",
                "db_user": "my_db_user",
                "dbfilter": "",
            },
        )

    def test_db_profile_clever_direct(self):
        parser = create_config(["test_db_clever_direct"])
        assertParser(
            self,
            parser,
            {
                "db_host": "my_db_host_direct",
                "db_maxconn": 64,
                "db_name": "my_db_name_direct",
                "db_password": "my_db_password_direct",
                "db_port": 4567,
                "db_sslmode": "prefer",
                "db_template": "template0",
                "db_user": "my_db_user_direct",
                "dbfilter": "",
            },
        )

    def test_db_profile_clever_full_both(self):
        parser = create_config(["test_db_clever", "test_db_clever_direct"])
        assertParser(
            self,
            parser,
            {
                "db_host": "my_db_host_direct",
                "db_maxconn": 64,
                "db_name": "my_db_name_direct",
                "db_password": "my_db_password_direct",
                "db_port": 4567,
                "db_sslmode": "prefer",
                "db_template": "template0",
                "db_user": "my_db_user_direct",
                "dbfilter": "",
            },
        )

    def test_filestore(self):
        """
        Assert module `odoo_filestore_s3` is enable.
        Returns:

        """
        parser = create_config(["filestore_s3"])
        assertParser(
            self,
            parser,
            {
                "server_wide_modules": ["odoo_filestore_s3"],
            },
        )
        parser = create_config(["filestore_s3_cellar"])
        assertParser(
            self,
            parser,
            {
                "server_wide_modules": ["odoo_filestore_s3"],
            },
        )
